# -*- coding: utf-8 -*-

from django.conf import settings


DJTASKS_DELETE_INTERVAL = getattr(settings, "DJTASKS_DELETE_INTERVAL", 14)

DJTASKS_LOCK = getattr(settings, "DJTASKS_LOCK", True)
DJTASKS_LOCK_ID = getattr(settings, "DJTASKS_LOCK_ID", "")

DJTASKS_SLEEP_INTERVAL = getattr(settings, "DJTASKS_SLEEP_INTERVAL", 10)

DJTASKS_TASKS = getattr(settings, "DJTASKS_TASKS", [])
DJTASKS_TASKS += ["dj_tasks.tasks.DeleteOldTaskRunTask"]



# -*- coding: utf-8 -*-

import socket
import sys
import time

from django.core.management.base import BaseCommand
from django.db import connection

from dj_tasks import settings as dj_tasks_settings
from dj_tasks.utils import get_class


class Command(BaseCommand):

    help = "run all specified tasks in dj_tasks_settings.DJTASKS_TASKS"

    def add_arguments(self, parser):
        parser.add_argument(
            "--production",
            action="store_true",
            dest="production",
            default=False,
            help="Specify when on production server.",
        )

    def get_lock_id(self):
        if dj_tasks_settings.DJTASKS_LOCK_ID:
            return dj_tasks_settings.DJTASKS_LOCK_ID
        raise NotImplementedError("Specify DJTASKS_LOCK_ID in settings")

    def get_tasks(self):
        tasks = []
        for task in dj_tasks_settings.DJTASKS_TASKS:
            tasks.append(get_class(task))
        return tasks

    def handle(self, *args, **options):
        if dj_tasks_settings.DJTASKS_LOCK:
            if not self.is_lock_free(options["production"]):
                sys.exit()

        while True:
            for Task in self.get_tasks():
                task = Task(self.output)
                task.full_run()

            # clone database connection
            connection.close()
            self.output("Database connection closed.")

            # no need to run more frequently than the tasks require
            time.sleep(dj_tasks_settings.DJTASKS_SLEEP_INTERVAL)

    def is_lock_free(self, production):
        global lock_socket
        lock_socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)

        lock_id = self.get_lock_id()
        if production:
            lock_id += ".production"

        try:
            lock_socket.bind("\0" + lock_id)
            self.stdout.write(
                self.style.SUCCESS(f"Acquired lock {lock_id}\n")
            )
            return True
        except socket.error:
            # socket already locked, task must already be running
            self.stdout.write(
                self.style.WARNING("Failed to acquire lock\n")
            )
            return False

    def output(self, message, style="SUCCESS"):
        style = getattr(self.style, style)
        self.stdout.write(style(message))



# -*- coding: utf-8 -*-

from django.utils import timezone

from dj_tasks import settings as dj_tasks_settings

from .models import Task as TaskModel, TaskRun


class Task:

    name = None
    frequency = None

    def __init__(self, output):
        self.output = output

    def full_run(self):
        instance = self.get_instance()
        if instance.run_again():
            self.output(f"Running {self.get_name()}")
            self.run()
            TaskRun.objects.create(task=instance)
            self.output(f"Completed {self.get_name()}\r")

    def get_frequency(self):
        if self.frequency:
            return self.frequency
        raise NotImplementedError("Every task needs a frequency.")

    def get_instance(self):
        task, c = TaskModel.objects.update_or_create(
            name=self.get_name(), defaults={"frequency": self.get_frequency()}
        )
        return task

    def get_name(self):
        if self.name:
            return self.name
        raise NotImplementedError("Every task needs a name.")

    def run(self):
        raise NotImplementedError("Every task needs a run method.")


class DeleteOldTaskRunTask(Task):

    name = "Delete old TaskRun entries"
    frequency = 60 * 60 * 24  # daily

    def run(self):
        interval = timezone.timedelta(
            days=dj_tasks_settings.DJTASKS_DELETE_INTERVAL
        )
        task_runs = TaskRun.objects.filter(created__lte=timezone.now() - interval)
        if task_runs.exists():
            task_runs.delete()



# -*- coding: utf-8 -*-

from django.db import models
from django.utils import timezone

from model_utils.models import TimeStampedModel


class Task(TimeStampedModel):

    name = models.CharField(max_length=120, unique=True)
    frequency = models.PositiveIntegerField()

    def __str__(self):
        return self.name

    def get_last_run(self):
        return self.taskrun_set.order_by("-created").first()

    def run_again(self):
        last_run = self.get_last_run()
        if last_run:
            last_run_time = last_run.created
            next_run_time = last_run_time + timezone.timedelta(
                seconds=self.frequency
            )
            return next_run_time <= timezone.now()
        return True


class TaskRun(TimeStampedModel):

    task = models.ForeignKey("Task", on_delete=models.CASCADE)



# -*- coding: utf-8 -*-

from django.contrib import admin

from .models import Task, TaskRun


@admin.register(Task)
class TaskAdmin(admin.ModelAdmin):
    pass


@admin.register(TaskRun)
class TaskRunAdmin(admin.ModelAdmin):
    pass



# Generated by Django 3.0.7 on 2020-06-05 14:44

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=120, unique=True)),
                ('frequency', models.PositiveIntegerField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TaskRun',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dj_tasks.Task')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-

from importlib import import_module


def get_class(dot_path):
    # get path
    dot_path = dot_path.split(".")
    path = ".".join(dot_path[:-1])

    # get class name
    dot_path.reverse()
    class_name = dot_path[0]

    # return Class
    module = import_module(path)
    return getattr(module, class_name)
